import 'dart:io';
import 'dart:convert' show utf8, jsonDecode;

import 'package:bottom_line/json_tool.dart';
import 'package:resource/resource.dart';

import 'package:keychain/src/keychain_exception.dart';
import 'package:keychain/src/keychain_light.dart';
import 'package:keychain/src/keychain_cache.dart';

import 'keychain_exception.dart';
import 'keychain_exception.dart';


abstract class Keychain extends KeychainLight {

  
  static String fromString(String string) {
    return KeychainLight.fromString(string);
  }
  
  static String fromEnv(String envVar, {bool allowNull: false}) {
    if (_cache.environment[envVar] != null || allowNull) return _cache.environment[envVar];
    var result = Platform.environment[envVar];
    if (result == null && !allowNull) throw KeychainExeception('The environment variable "$envVar" was not found.');
    _cache.environment[envVar] = result;
    return result;
  }

  static String fromEnvJson(String key, String envVar, {bool allowNull: false}) {
    final cachedKey = key + '|' + envVar;
    final cached = _cache.envJson[cachedKey];
    if (cached != null || allowNull) {
      return cached;
    }
    var obj = _cache.environment[envVar];
    if (obj == null) {
      obj = Platform.environment[envVar];
      if (obj == null && !allowNull) {
        throw KeychainExeception('The environment variable "$envVar" was not found.');
      } else if (obj == null && allowNull) {
        return null;
      } else {
        _cache.environment[envVar] = obj;
      }
    }
    final map = jsonDecode(obj);
    final r = MapTool(map).find(key);
    if (r == null && !allowNull) {
      throw KeychainExeception('The key "$key" is null.');
    }
    _cache.envJson[cachedKey] = r.toString();
    return _cache.envJson[cachedKey];

  }

  static String fromJsonFile(String key, String filePath, {bool allowNull: false}) {
    var varId = key + '|' + filePath;
    if (_cache.fileVar[varId] != null) return _cache.fileVar[varId];
    if (_cache.fileMap[filePath] == null) {
      if (filePath.startsWith(Platform.pathSeparator)) {
        filePath = filePath.substring(1);
      }
      var targetPath = filePath.split(Platform.pathSeparator);
      var scriptPath = Platform.script.pathSegments;
      var index = scriptPath.indexOf(targetPath[0]);
      if (index == -1) {
        throw KeychainExeception('Json file target must relative to a directory on the script path.');
      }
      var pathCopy = List.of(scriptPath);
      pathCopy.replaceRange(index, pathCopy.length, targetPath);
      var u = Uri.file(Platform.pathSeparator + pathCopy.join(Platform.pathSeparator), windows: Platform.isWindows);
      var str = File.fromUri(u).readAsStringSync();
      _cache.fileMap[filePath] = jsonDecode(str);
    }
    var r = MapTool(_cache.fileMap[filePath]).find(key);
    if (r == null && !allowNull) {
      throw KeychainExeception('Key "$key" is null which is not allowed. To allow null, set allowNull: true');
    }
    _cache.fileVar[varId] = r;
    return r;
    
  }

  static Future<String> fromJsonResource(String key,
                             String resource,
                             {bool allowNull: false}) 
                             async {
    var varId = key + '|' + resource;
    if (_cache.fileVar[varId] != null) return _cache.fileVar[varId];
    if (_cache.fileMap[resource] == null) {
      var r = Resource(resource);
      var str = await r.readAsString();
      _cache.fileMap[resource] = jsonDecode(str);
    }
    var r = MapTool(_cache.fileMap[resource]).find(key);
    if (r == null && !allowNull) {
      throw KeychainExeception('Key "$key" is null which is not allowed. To allow null, set allowNull: true');
    }
    _cache.fileVar[varId] = r;
    return r;
  }

  static String cacheValue(String key, String value) {
    if (_cache.user[key] != null) return _cache.user[key];
    _cache.user[key] = value;
    return value;
  }

  static final _cache = KeychainCache();
  static final environment = Platform.environment;

}